@@ -19,6 +19,7 @@
  * applicable instead of those above.
  */
 
+#include <algorithm>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -46,34 +47,35 @@
 namespace WPS8ParserInternal
 {
 //! Internal: the subdocument of a WPS8Parser
-class SubDocument : public WPSTextSubDocument
+class SubDocument final : public WPSTextSubDocument
 {
 public:
 	//! type of an entry
 	enum Type { Unknown, TEXT };
 	//! constructor for a text entry
-	SubDocument(RVNGInputStreamPtr input, WPS8Parser &pars, WPSEntry const &entry) :
-		WPSTextSubDocument(input, &pars), m_entry(entry) {}
+	SubDocument(RVNGInputStreamPtr const &input, WPS8Parser &pars, WPSEntry const &entry)
+		: WPSTextSubDocument(input, &pars)
+		, m_entry(entry) {}
 	//! destructor
-	~SubDocument() {}
+	~SubDocument() final {}
 
 	//! operator==
-	virtual bool operator==(shared_ptr<WPSTextSubDocument> const &doc) const
+	bool operator==(std::shared_ptr<WPSSubDocument> const &doc) const final
 	{
 		if (!doc || !WPSTextSubDocument::operator==(doc))
 			return false;
-		SubDocument const *sDoc = dynamic_cast<SubDocument const *>(doc.get());
+		auto const *sDoc = dynamic_cast<SubDocument const *>(doc.get());
 		if (!sDoc) return false;
 		return m_entry == sDoc->m_entry;
 	}
 
 	//! the parser function
-	void parse(shared_ptr<WPSContentListener> &listener, libwps::SubDocumentType subDocumentType);
+	void parse(std::shared_ptr<WPSContentListener> &listener, libwps::SubDocumentType subDocumentType) final;
 	//! the entry
 	WPSEntry m_entry;
 };
 
-void SubDocument::parse(shared_ptr<WPSContentListener> &listener, libwps::SubDocumentType subDocumentType)
+void SubDocument::parse(std::shared_ptr<WPSContentListener> &listener, libwps::SubDocumentType subDocumentType)
 {
 	if (!listener.get())
 	{
@@ -86,7 +88,7 @@ void SubDocument::parse(shared_ptr<WPSContentListener> &listener, libwps::SubDoc
 		return;
 	}
 
-	WPSContentListenerPtr &listen =  static_cast<WPSContentListenerPtr &>(listener);
+	auto &listen =  static_cast<WPSContentListenerPtr &>(listener);
 	if (!m_parser)
 	{
 		listen->insertCharacter(' ');
@@ -116,7 +118,7 @@ void SubDocument::parse(shared_ptr<WPSContentListener> &listener, libwps::SubDoc
 	}
 
 
-	WPS8Parser *mnParser = dynamic_cast<WPS8Parser *>(m_parser);
+	auto *mnParser = dynamic_cast<WPS8Parser *>(m_parser);
 	if (!mnParser)
 	{
 		WPS_DEBUG_MSG(("WPS8ParserInternal::SubDocument::parse: can not find a WPS8Parser...\n"));
@@ -147,7 +149,7 @@ struct Frame
 	 * - Table : a table
 	 * - Object : a picture, ...
 	 * - Text : a text zone */
-	enum { UNKNOWN = 0, Header, Footer, Table, Object, Text };
+	enum { UNKNOWN = 0, DbField, Header, Footer, Table, Object, Text };
 
 	//! a flag to know if the frame is already sent to the listener
 	mutable bool m_parsed;
@@ -174,6 +176,9 @@ std::ostream &operator<<(std::ostream &o, Frame const &ft)
 {
 	switch (ft.m_type)
 	{
+	case Frame::DbField:
+		o<< "db[field]";
+		break;
 	case Frame::Header:
 		o<< "header";
 		break;
@@ -225,10 +230,19 @@ std::ostream &operator<<(std::ostream &o, Frame const &ft)
 //! the state of WPS8
 struct State
 {
-	State() : m_eof(-1), m_pageSpan(), m_localeLanguage(""), m_background(),
-		m_frameList(), m_object2FrameMap(), m_table2FrameMap(),
-		m_docPropertyTypes(), m_frameTypes(),
-		m_numColumns(1), m_actPage(0), m_numPages(0)
+	State()
+		: m_eof(-1)
+		, m_pageSpan()
+		, m_localeLanguage("")
+		, m_background()
+		, m_frameList()
+		, m_object2FrameMap()
+		, m_table2FrameMap()
+		, m_docPropertyTypes()
+		, m_frameTypes()
+		, m_numColumns(1)
+		, m_actPage(0)
+		, m_numPages(0)
 	{
 		initTypeMaps();
 	}
@@ -269,19 +283,19 @@ void State::initTypeMaps()
 		0x26, 0x22, 0x27, 0x22,
 		0x28, 0x22, 0x29, 0x22, 0x2a, 0x22, 0x2b, 0x22, 0x2c, 0x82, 0x2d, 0x82, 0x2e, 0x82
 	};
-	for (int i = 0; i+1 < int(sizeof(docTypes)/sizeof(int)); i+=2)
+	for (int i = 0; i+1 < int(WPS_N_ELEMENTS(docTypes)); i+=2)
 		m_docPropertyTypes[docTypes[i]] = docTypes[i+1];
 	static int const frameTypes[] =
 	{
 		0, 0x1a, 1, 0x12, 2, 0x12, 3, 0x2, 4, 0x22, 5, 0x22, 6, 0x22, 7, 0x22,
 		8, 0x22, 9, 0x22, 0xa, 0x22,
-		0x10, 0x2a, 0x11, 0x82, 0x13, 0x12, 0x14, 0x12, 0x17, 0x2,
+		0x10, 0x2a, 0x11, 0x82, 0x13, 0x12, 0x14, 0x12, 0x16, 0x2a, 0x17, 0x2,
 		0x18, 0x22, 0x19, 0x22, 0x1a, 0x12, 0x1b, 0x22, 0x1d, 0x22, 0x1e, 0x22, 0x1f, 0x22,
 		0x20, 0x22, 0x26, 0x22,
 		0x2a, 0x22, 0x2c, 0x1a, 0x2d, 0x1a, 0x2e, 0x22, 0x2f, 0x2,
 		0x30, 0x22
 	};
-	for (int i = 0; i+1 < int(sizeof(frameTypes)/sizeof(int)); i+=2)
+	for (int i = 0; i+1 < int(WPS_N_ELEMENTS(frameTypes)); i+=2)
 		m_frameTypes[frameTypes[i]] = frameTypes[i+1];
 
 }
@@ -289,8 +303,8 @@ void State::initTypeMaps()
 class TableRecursionGuard
 {
 	// disable copying
-	TableRecursionGuard(const TableRecursionGuard &);
-	TableRecursionGuard &operator=(const TableRecursionGuard &);
+	TableRecursionGuard(const TableRecursionGuard &) = delete;
+	TableRecursionGuard &operator=(const TableRecursionGuard &) = delete;
 
 public:
 	TableRecursionGuard(std::deque<int> &stack, const int id)
@@ -312,10 +326,14 @@ private:
 }
 
 // constructor, destructor
-WPS8Parser::WPS8Parser(RVNGInputStreamPtr &input, WPSHeaderPtr &header) :
-	WPSParser(input, header),
-	m_listener(), m_graphParser(), m_tableParser(), m_textParser(), m_state(),
-	m_sendingTables()
+WPS8Parser::WPS8Parser(RVNGInputStreamPtr &input, WPSHeaderPtr &header)
+	: WPSParser(input, header)
+	, m_listener()
+	, m_graphParser()
+	, m_tableParser()
+	, m_textParser()
+	, m_state()
+	, m_sendingTables()
 {
 	if (version()<5) setVersion(5);
 	m_state.reset(new WPS8ParserInternal::State);
@@ -358,7 +376,7 @@ bool WPS8Parser::checkInFile(long pos)
 }
 
 // listener, new page
-void WPS8Parser::setListener(shared_ptr<WPSContentListener> listener)
+void WPS8Parser::setListener(std::shared_ptr<WPSContentListener> const &listener)
 {
 	m_listener = listener;
 	m_graphParser->setListener(m_listener);
@@ -366,16 +384,16 @@ void WPS8Parser::setListener(shared_ptr<WPSContentListener> listener)
 	m_textParser->setListener(m_listener);
 }
 
-shared_ptr<WPSContentListener> WPS8Parser::createListener(librevenge::RVNGTextInterface *interface)
+std::shared_ptr<WPSContentListener> WPS8Parser::createListener(librevenge::RVNGTextInterface *interface)
 {
 	std::vector<WPSPageSpan> pageList;
 	WPSPageSpan ps(m_state->m_pageSpan);
 
 	int numPages = 1;
-	for (size_t i = 0; i < m_state->m_frameList.size(); i++)
+	for (auto const &frame : m_state->m_frameList)
 	{
-		if (m_state->m_frameList[i].m_pos.page() > numPages)
-			numPages = m_state->m_frameList[i].m_pos.page();
+		if (frame.m_pos.page() > numPages)
+			numPages = frame.m_pos.page();
 	}
 
 	int textPages = m_textParser->numPages();
@@ -408,7 +426,7 @@ shared_ptr<WPSContentListener> WPS8Parser::createListener(librevenge::RVNGTextIn
 	pageList.push_back(ps);
 	for (int i = 1; i < numPages; i++) pageList.push_back(ps);
 	m_state->m_numPages=numPages;
-	return shared_ptr<WPSContentListener>
+	return std::shared_ptr<WPSContentListener>
 	       (new WPSContentListener(pageList, interface));
 }
 
@@ -421,7 +439,7 @@ void WPS8Parser::newPage(int number)
 	{
 		m_state->m_actPage++;
 
-		if (m_listener.get() == 0L || m_state->m_actPage == 1)
+		if (m_listener.get() == nullptr || m_state->m_actPage == 1)
 			continue;
 		m_listener->insertBreak(WPS_PAGE_BREAK);
 		m_graphParser->sendObjects(m_state->m_actPage);
@@ -481,14 +499,14 @@ bool WPS8Parser::sendTable(Vec2f const &size, int objectId)
 		return false;
 	}
 
-	std::map<int, int>::iterator pos = m_state->m_object2FrameMap.find(objectId);
+	auto pos = m_state->m_object2FrameMap.find(objectId);
 	if (pos == m_state->m_object2FrameMap.end())
 	{
 		WPS_DEBUG_MSG(("WPS8Parser::sendTable can not find the table %d \n", objectId));
 		return false;
 	}
 
-	WPS8ParserInternal::Frame const &frame = m_state->m_frameList[(size_t)pos->second];
+	auto const &frame = m_state->m_frameList[(size_t)pos->second];
 	if (frame.m_idStrs < 0)
 	{
 		WPS_DEBUG_MSG(("WPS8Parser:sendTable can not find the text zone \n"));
@@ -512,11 +530,11 @@ bool WPS8Parser::sendTable(Vec2f const &size, int objectId)
 
 int WPS8Parser::getTableSTRSId(int tableId) const
 {
-	std::map<int, int>::iterator pos = m_state->m_table2FrameMap.find(tableId);
+	auto pos = m_state->m_table2FrameMap.find(tableId);
 	// probably ok: checkme
 	if (pos == m_state->m_table2FrameMap.end())
 		return -1;
-	WPS8ParserInternal::Frame const &frame = m_state->m_frameList[(size_t)pos->second];
+	auto const &frame = m_state->m_frameList[(size_t)pos->second];
 	if (frame.m_idStrs < 0)
 	{
 		WPS_DEBUG_MSG(("WPS8Parser:getTableSTRSId can not find the text zone \n"));
@@ -600,10 +618,8 @@ bool WPS8Parser::createStructures()
 	m_graphParser->readStructures(input);
 	m_tableParser->readStructures(input);
 
-	NameMultiMap::iterator pos;
-
 	// read DOP zone (document properties)
-	pos = getNameEntryMap().lower_bound("DOP ");
+	auto pos = getNameEntryMap().lower_bound("DOP ");
 	while (getNameEntryMap().end() != pos)
 	{
 		WPSEntry const &entry = pos++->second;
@@ -659,6 +675,16 @@ bool WPS8Parser::createStructures()
 
 		readFRAM(entry);
 	}
+	// FRCD
+	pos = getNameEntryMap().lower_bound("FRCD");
+	while (getNameEntryMap().end() != pos)
+	{
+		WPSEntry const &entry = pos++->second;
+		if (!entry.hasName("FRCD")) break;
+		if (!entry.hasType("FRCD")) continue;
+		readFRCD(entry);
+	}
+
 	// creates the correspondance between the eobj and the frame
 	size_t numFrames = m_state->m_frameList.size();
 	for (size_t i = 0; i < numFrames; i++)
@@ -669,7 +695,6 @@ bool WPS8Parser::createStructures()
 		m_state->m_table2FrameMap[frame.m_idTable] = (int) i;
 	}
 
-
 	m_graphParser->computePositions();
 
 	return true;
@@ -681,20 +706,17 @@ bool WPS8Parser::createOLEStructures()
 	if (!input) return false;
 
 	if (!input->isStructured()) return true;
-	WPSOLEParser oleParser("CONTENTS");
+	WPSOLEParser oleParser("CONTENTS", libwps_tools_win::Font::WIN3_WEUROPE);
 	if (!oleParser.parse(input)) return false;
 
-	m_graphParser->storeObjects(oleParser.getObjects(),oleParser.getObjectsId());
+	m_graphParser->storeObjects(oleParser.getObjectsMap());
 #ifdef DEBUG
 	// there can remain some embedded Works subdocument ( WKS, ... )
 	// with name MN0 and some unknown picture ole
-	std::vector<std::string> unparsed = oleParser.getNotParse();
+	auto unparsed = oleParser.getNotParse();
 
-	size_t numUnparsed = unparsed.size();
-
-	for (size_t i = 0; i < numUnparsed; i++)
+	for (std::string const &name : unparsed)
 	{
-		std::string const &name = unparsed[i];
 		if (name == "CONTENTS")
 			continue;
 		if (name== "SPELLING")
@@ -729,14 +751,11 @@ void WPS8Parser::sendPageFrames()
 {
 	if (!m_listener) return;
 
-	typedef WPS8ParserInternal::Frame Frame;
 	RVNGInputStreamPtr input = getInput();
 	long actPos = input->tell();
 
-	size_t numFrames = m_state->m_frameList.size();
-	for (size_t i = 0; i < numFrames; i++)
+	for (auto const &frame : m_state->m_frameList)
 	{
-		Frame const &frame = m_state->m_frameList[i];
 		if (frame.m_parsed) continue;
 		if (frame.m_pos.page() < 0 || frame.m_idObject != -1) continue;
 		if (frame.m_pos.size()[0] <= 0 || frame.m_pos.size()[1] <= 0) continue;
@@ -744,6 +763,7 @@ void WPS8Parser::sendPageFrames()
 		WPSPosition pos(frame.m_pos);
 		if (pos.origin().y() < 0 || pos.origin().x() < 0)
 		{
+			if (frame.m_type == WPS8ParserInternal::Frame::DbField) continue;
 			// FIXME: the
 			WPS_DEBUG_MSG(("WPS8Parser::insertPageFrames: origin is too small\n"));
 			Vec2f orig = pos.origin();
@@ -754,16 +774,16 @@ void WPS8Parser::sendPageFrames()
 		pos.m_anchorTo = WPSPosition::Page;
 		pos.m_wrapping = WPSPosition::WDynamic;
 		frame.m_parsed = true;
-		if (frame.m_type == Frame::Object)
+		if (frame.m_type == WPS8ParserInternal::Frame::Object)
 			m_graphParser->sendObject(pos, frame.m_idOle, true);
-		else if (frame.m_type == Frame::Text)
+		else if (frame.m_type == WPS8ParserInternal::Frame::Text)
 		{
 			librevenge::RVNGPropertyList frameExtras;
 			if (!frame.m_backgroundColor.isWhite())
 				frameExtras.insert("fo:background-color", frame.m_backgroundColor.str().c_str());
 			sendTextBox(pos, frame.m_idStrs, frameExtras);
 		}
-		else if (frame.m_type != Frame::Header && frame.m_type != Frame::Footer)
+		else if (frame.m_type != WPS8ParserInternal::Frame::Header && frame.m_type != WPS8ParserInternal::Frame::Footer)
 		{
 			WPS_DEBUG_MSG(("WPS8Parser::insertPageFrames: not implemented type%d\n", int(frame.m_type)));
 		}
@@ -826,11 +846,11 @@ bool WPS8Parser::parseHeaderIndexEntry()
 
 	f << "Entries("<<name<<")";
 	if (cch != 24) f << ", #size=" << int(cch);
-	int id = (int) libwps::readU16(input);
+	auto id = (int) libwps::readU16(input);
 	f << ", id=" << id << ", (";
 	for (i = 0; i < 2; i ++)
 	{
-		int val = (int) libwps::read16(input);
+		auto val = (int) libwps::read16(input);
 		f << val << ",";
 	}
 
@@ -891,7 +911,7 @@ bool WPS8Parser::parseHeaderIndexEntryEnd(long endPos, WPSEntry &hie, std::strin
 	long len = endPos-pos;
 	libwps::DebugStream f;
 
-	int size = (int) libwps::read16(input);
+	auto size = (int) libwps::read16(input);
 	librevenge::RVNGString str;
 	if (2*(size+1) != len || !m_textParser->readString(input, 2*size, str))
 	{
@@ -920,13 +940,13 @@ bool WPS8Parser::parseHeaderIndex()
 	input->seek(0x08, librevenge::RVNG_SEEK_SET);
 
 	long pos = input->tell();
-	int i0 = (int) libwps::read16(input);
-	int i1 = (int) libwps::read16(input);
+	auto i0 = (int) libwps::read16(input);
+	auto i1 = (int) libwps::read16(input);
 	uint16_t n_entries = libwps::readU16(input);
 	// fixme: sanity check n_entries
 
 	libwps::DebugStream f;
-	f << "Header: N=" << n_entries << ", " << i0 << ", " << i1 << "(";
+	f << "Entries(HeaderZ): N=" << n_entries << ", " << i0 << ", " << i1 << "(";
 
 	for (int i = 0; i < 4; i++)
 		f << std::hex << libwps::read16(input) << std::dec << ",";
@@ -947,7 +967,7 @@ bool WPS8Parser::parseHeaderIndex()
 		uint16_t unknown1 = libwps::readU16(input); // function of the size of the entries ?
 
 		uint16_t n_entries_local = libwps::readU16(input);
-		f << "Header("<<std::hex << unknown1<<"): N=" << std::dec << n_entries_local;
+		f << "HeaderZ("<<std::hex << unknown1<<"): N=" << std::dec << n_entries_local;
 
 		if (n_entries_local > 0x20)
 		{
@@ -1031,12 +1051,10 @@ bool WPS8Parser::readDocProperties(WPSEntry const &entry, WPSPageSpan &page)
 	std::string error;
 	bool readOk = readBlockData(input, endPage,mainData, error);
 
-	size_t numChild = mainData.m_recursData.size();
 	double dim[8] = {0, 0, 0, 0, 0, 0, 0, 0};
 	bool setVal[8] = { false, false, false, false, false, false, false, false};
-	for (size_t c = 0; c < numChild; c++)
+	for (auto const &dt : mainData.m_recursData)
 	{
-		WPS8Struct::FileData const &dt = mainData.m_recursData[c];
 		if (dt.isBad()) continue;
 		if (m_state->m_docPropertyTypes.find(dt.id())==m_state->m_docPropertyTypes.end())
 		{
@@ -1122,7 +1140,7 @@ bool WPS8Parser::readDocProperties(WPSEntry const &entry, WPSPageSpan &page)
 				ok = false;
 				break;
 			}
-			int size = int(dt.end()-dt.begin());
+			auto size = int(dt.end()-dt.begin());
 			if (size < 2 || size%2 != 0)
 			{
 				ok = false;
@@ -1157,7 +1175,7 @@ bool WPS8Parser::readDocProperties(WPSEntry const &entry, WPSPageSpan &page)
 			elt=0;
 			while (elt++ < numElt)
 			{
-				long val = (long) libwps::readU16(input);
+				auto val = (long) libwps::readU16(input);
 				if (val) f2 << "f" << elt << "=" << std::hex << val << std::dec << ",";
 			}
 			f2 << "],";
@@ -1173,7 +1191,7 @@ bool WPS8Parser::readDocProperties(WPSEntry const &entry, WPSPageSpan &page)
 				ok = false;
 				break;
 			}
-			int size = int(dt.end()-dt.begin());
+			auto size = int(dt.end()-dt.begin());
 			if (size != 0x100)
 			{
 				ok = false;
@@ -1184,7 +1202,7 @@ bool WPS8Parser::readDocProperties(WPSEntry const &entry, WPSPageSpan &page)
 			f2 << "unk" << std::hex << dt.id() << "=" << "[";
 			for (int i = 0; i < 128; i++)
 			{
-				long val = (long) libwps::readU16(input);
+				auto val = (long) libwps::readU16(input);
 				if (val) f2 << "f" << i << "=" << std::hex << val << std::dec << ",";
 			}
 			f2 << "]" << std::dec << ",";
@@ -1314,7 +1332,6 @@ bool WPS8Parser::readFRAM(WPSEntry const &entry)
 		std::string error;
 		bool readOk = readBlockData(input, lastPos + sz, mainData, error);
 
-		size_t numChild = mainData.m_recursData.size();
 		double dim[3] = {0, 0, 0};
 		bool setVal[3] = { false, false, false};
 
@@ -1325,9 +1342,8 @@ bool WPS8Parser::readFRAM(WPSEntry const &entry)
 		Frame frame;
 		int lastPage = 0;
 		Vec2f minP, sizeP;
-		for (size_t c = 0; c < numChild; c++)
+		for (auto const &dt : mainData.m_recursData)
 		{
-			WPS8Struct::FileData const &dt = mainData.m_recursData[c];
 			if (dt.isBad()) continue;
 			if (m_state->m_frameTypes.find(dt.id())==m_state->m_frameTypes.end())
 			{
@@ -1384,6 +1400,10 @@ bool WPS8Parser::readFRAM(WPSEntry const &entry)
 			case 1:
 				switch (dt.m_value)
 				{
+				// associated with mailing database with 0x16
+				case 4:
+					frame.m_type = Frame::DbField;
+					break;
 				// associated with 0x13=22:[0|96], 0x18,1b=0x41,0x2e, rarely with 0x1a
 				case 6:
 					frame.m_type = Frame::Header;
@@ -1410,7 +1430,7 @@ bool WPS8Parser::readFRAM(WPSEntry const &entry)
 					break;
 
 				default:
-					f2 << "###type=" << dt.m_value;
+					f2 << "###type=" << dt.m_value << ",";
 					break;
 				}
 				break;
@@ -1435,7 +1455,7 @@ bool WPS8Parser::readFRAM(WPSEntry const &entry)
 					ok = false;
 					break;
 				}
-				int size = int(dt.end()-dt.begin());
+				auto size = int(dt.end()-dt.begin());
 				if (size < 2 || (size+2)%4 != 0)
 				{
 					ok = false;
@@ -1468,6 +1488,9 @@ bool WPS8Parser::readFRAM(WPSEntry const &entry)
 				input->seek(actPos, librevenge::RVNG_SEEK_SET);
 				break;
 			}
+			case 0x16: // associated with field and database
+				f2 << "database=" << dt.m_text << ":" << dt.m_value << ",";
+				break;
 			// associated with a tex box and field 0x19
 			case 0x17:
 				color = dt.isTrue();
@@ -1627,6 +1650,189 @@ bool WPS8Parser::readFRAM(WPSEntry const &entry)
 	return false;
 }
 
+/** try to read a frame XXX database  */
+bool WPS8Parser::readFRCD(WPSEntry const &entry)
+{
+	libwps::DebugStream f;
+	if (!entry.hasType(entry.name()))
+	{
+		WPS_DEBUG_MSG(("WPS8Parser::readFRCD warning: name=%s, type=%s\n",
+		               entry.name().c_str(), entry.type().c_str()));
+		return false;
+	}
+	RVNGInputStreamPtr input = getInput();
+
+	long page_offset = entry.begin();
+	long length = entry.length();
+	long endPage = entry.end();
+
+	if (length < 4+33+5*8)
+	{
+		WPS_DEBUG_MSG(("WPS8Parser::readFRCD warning: length=0x%lx\n", (unsigned long) length));
+		return false;
+	}
+	entry.setParsed();
+	input->seek(page_offset, librevenge::RVNG_SEEK_SET);
+	auto val=int(libwps::readU16(input)); // 0x30
+	if (val!=0x30) f << "f0=" << val << ",";
+	auto sSz=int(libwps::readU16(input));
+	librevenge::RVNGString str;
+	if (4+2*sSz+33+5*8>length || !m_textParser->readString(input, sSz, str))
+	{
+		WPS_DEBUG_MSG(("WPS8Parser::readFRCD warning: can not read main string\n"));
+		return false;
+	}
+	f << str.cstr() << ",";
+	for (int i=0; i<16; ++i)   // f1=10,f2=22,f4=4,f7=256,f11=-1,f12=-256,f13=255,f14=-1
+	{
+		val=int(libwps::read16(input));
+		if (val) f << "f" << i+1 << "=" << val << ",";
+	}
+	val=int(libwps::read8(input));
+	if (val) f << "g0=" << val << ",";
+	ascii().addPos(page_offset);
+	ascii().addNote(f.str().c_str());
+	for (int i=0; i<5; ++i)
+	{
+		long pos=input->tell();
+		f.str("");
+		f << "FRCD-A" << i << ":";
+		if (pos+8>endPage)
+		{
+			WPS_DEBUG_MSG(("WPS8Parser::readFRCD warning: the zone seems too short\n"));
+			f << "###";
+			ascii().addPos(pos);
+			ascii().addNote(f.str().c_str());
+			return true;
+		}
+		for (int s=0; s<2; ++s)
+		{
+			if (pos+4>endPage || libwps::read8(input)!=-1)
+			{
+				WPS_DEBUG_MSG(("WPS8Parser::readFRCD warning: the zone seems bad\n"));
+				f << "###";
+				ascii().addPos(pos);
+				ascii().addNote(f.str().c_str());
+				return true;
+			}
+			val=int(libwps::read8(input));
+			if (val==-1)
+			{
+				val=int(libwps::read16(input));
+				if (val!=0x204)
+					f << "f" << s << "=" << val << ",";
+			}
+			else if (val==-2)
+			{
+				val=int(libwps::read8(input));
+				if (val!=-1) f << "g" << s << "=" << val << ",";
+				sSz=int(libwps::readU8(input));
+				str.clear();
+				if (pos+2*sSz>endPage || (sSz && !m_textParser->readString(input, 2*sSz, str)))
+				{
+					WPS_DEBUG_MSG(("WPS8Parser::readFRCD warning: the zone seems too short\n"));
+					f << "###";
+					ascii().addPos(pos);
+					ascii().addNote(f.str().c_str());
+					return true;
+				}
+				if (!str.empty())
+					f << str.cstr() << ",";
+			}
+			else
+			{
+				WPS_DEBUG_MSG(("WPS8Parser::readFRCD warning: find unexpected value\n"));
+				f << "###val=" << val << ",";
+				ascii().addPos(pos);
+				ascii().addNote(f.str().c_str());
+				return true;
+			}
+		}
+		ascii().addPos(pos);
+		ascii().addNote(f.str().c_str());
+	}
+	long pos=input->tell();
+	f.str("");
+	f << "FRCD-B:";
+	if (pos+4>endPage)
+	{
+		WPS_DEBUG_MSG(("WPS8Parser::readFRCD warning: can not read the second zone\n"));
+		f << "###";
+		ascii().addPos(pos);
+		ascii().addNote(f.str().c_str());
+		return true;
+	}
+	auto N=int(libwps::readU16(input));
+	if (pos+4+10*N>endPage)
+	{
+		WPS_DEBUG_MSG(("WPS8Parser::readFRCD warning: can not read the second zone\n"));
+		f << "###";
+		ascii().addPos(pos);
+		ascii().addNote(f.str().c_str());
+		return true;
+	}
+	val=int(libwps::read16(input)); // 0
+	if (val) f << "f0=" << val << ",";
+	ascii().addPos(pos);
+	ascii().addNote(f.str().c_str());
+	for (int n=0; n<N; ++n)
+	{
+		pos=input->tell();
+		f.str("");
+		f << "FRCD-B" << n << ":";
+		if (pos+10>endPage)
+		{
+			WPS_DEBUG_MSG(("WPS8Parser::readFRCD warning: the zone seems too short\n"));
+			f << "###";
+			ascii().addPos(pos);
+			ascii().addNote(f.str().c_str());
+			return true;
+		}
+		val=int(libwps::read16(input)); // 0
+		if (val) f << "f0=" << val << ",";
+		for (int s=0; s<2; ++s)
+		{
+			if (pos+4>endPage || libwps::read8(input)!=-1)
+			{
+				WPS_DEBUG_MSG(("WPS8Parser::readFRCD warning: the second zone seems bad\n"));
+				f << "###";
+				ascii().addPos(pos);
+				ascii().addNote(f.str().c_str());
+				return true;
+			}
+			val=int(libwps::read8(input));
+			if (val==-2)
+			{
+				val=int(libwps::read8(input));
+				if (val!=-1) f << "g" << s << "=" << val << ",";
+				sSz=int(libwps::readU8(input));
+				str.clear();
+				if (pos+2*sSz>endPage || (sSz && !m_textParser->readString(input, 2*sSz, str)))
+				{
+					WPS_DEBUG_MSG(("WPS8Parser::readFRCD warning: the second zone seems too short\n"));
+					f << "###";
+					ascii().addPos(pos);
+					ascii().addNote(f.str().c_str());
+					return true;
+				}
+				if (!str.empty())
+					f << str.cstr() << ",";
+			}
+			else
+			{
+				WPS_DEBUG_MSG(("WPS8Parser::readFRCD warning: find unexpected value\n"));
+				f << "###val=" << val << ",";
+				ascii().addPos(pos);
+				ascii().addNote(f.str().c_str());
+				return true;
+			}
+		}
+		ascii().addPos(pos);
+		ascii().addNote(f.str().c_str());
+	}
+	return true;
+}
+
 // reads SYID zone: an unknown zone which seems to have the
 //    same number of entries than the text zone
 // also related to the MCLD ?
@@ -1705,10 +1911,10 @@ bool WPS8Parser::readWNPR(WPSEntry const &entry)
 
 	f << std::hex << libwps::readU32(input) << std::dec << ",";
 	long dim[4];
-	for (int i = 0; i < 4; i++)
+	for (long &i : dim)
 	{
-		dim[i] = libwps::read32(input);
-		if (dim[i] <= 0) return false;
+		i = libwps::read32(input);
+		if (i <= 0) return false;
 	}
 
 	if (dim[2] && dim[3])
@@ -1725,7 +1931,7 @@ bool WPS8Parser::readWNPR(WPSEntry const &entry)
 	f << "printmargins?=(";
 	for (int i = 0; i < 4; i++)
 	{
-		long val = (long) libwps::readU32(input);
+		auto val = (long) libwps::readU32(input);
 		long sz = dim[2+(i%2)];
 		if (sz)
 			f << float(val)/float(sz) << ",";
@@ -1770,8 +1976,8 @@ bool WPS8Parser::readWNPR(WPSEntry const &entry)
 	f << "paperSize=" << libwps::readU16(input) << ","; // 1: letter, 2:letter small, ..
 
 	long dim2[3];
-	for (int i = 0; i < 3; i++)
-		dim2[i] = libwps::read16(input);
+	for (long &i : dim2)
+		i = libwps::read16(input);
 	f << std::dec;
 	if (dim2[2] > 0)
 	{
@@ -1843,7 +2049,7 @@ bool WPS8Parser::readSPELLING(RVNGInputStreamPtr input, std::string const &oleNa
 {
 	// SPELLING
 	input->seek(0, librevenge::RVNG_SEEK_SET);
-	int vers = (int) libwps::read32(input); // always 6 ?
+	auto vers = (int) libwps::read32(input); // always 6 ?
 	if (vers < 0 || input->isEnd()) return false;
 
 	std::string fName = libwps::Debug::flattenFileName(oleName);
@@ -1859,7 +2065,7 @@ bool WPS8Parser::readSPELLING(RVNGInputStreamPtr input, std::string const &oleNa
 	while (!input->isEnd())
 	{
 		long pos = input->tell();
-		int numVal = (int) libwps::read32(input);
+		auto numVal = (int) libwps::read32(input);
 
 		if (numVal < 0 || input->seek(8*numVal, librevenge::RVNG_SEEK_CUR) || input->tell() != pos+4+8*numVal)
 		{
@@ -1881,7 +2087,7 @@ bool WPS8Parser::readSPELLING(RVNGInputStreamPtr input, std::string const &oleNa
 			}
 			else
 				id = listIds[ptr];
-			long sPos = (long) libwps::read32(input);
+			auto sPos = (long) libwps::read32(input);
 			f << std::hex << 2*sPos << ":SP" <<  std::dec << id << ",";
 		}
 		asciiFile.addPos(pos);
@@ -1891,10 +2097,10 @@ bool WPS8Parser::readSPELLING(RVNGInputStreamPtr input, std::string const &oleNa
 	f << "SPELLING:";
 	if (vers != 6) f << "version = " << vers << ", ";
 	f << "list=[";
-	for (std::map<uint32_t, int>::iterator it = listIds.begin(); it != listIds.end(); ++it)
+	for (auto it : listIds)
 	{
-		uint32_t val = it->first;
-		f << "SP" << std::dec << it->second << "(" << (val&0xFF) << ":"
+		uint32_t val = it.first;
+		f << "SP" << std::dec << it.second << "(" << (val&0xFF) << ":"
 		  << std::hex << (val>>8) << std::dec << "),";
 
 		// (val&0xFF) :121-123, 129, 133, 139, 145
